@@ -65,30 +65,34 @@ def emit_state(state):
 
 
 def try_upsert(collection, record_batch: RecordBatch, force=False):
-    if record_batch.length() >= record_batch.max_batch_size or force:
+    if record_batch.length() >= record_batch.max_batch_size or (force and record_batch.length() > 0):
         to_upsert = record_batch.flush()
         for record in to_upsert:
-            if record['_id']:
-                find_id = record['_id']
+            try:
+                if '_id' in record:
+                    find_id = record['_id']
 
-                # pop the key from update if primary key is _id
-                record.pop("_id")
+                    # pop the key from update if primary key is _id
+                    record.pop("_id")
 
-                # Last parameter True is upsert which inserts a new record if it doesnt exists or replaces current if found
-                collection.update_one({"_id": find_id}, {"$set": record}, True)
-            else:
-                # If no key property is available just insert the record as it is
-                collection.insert_one(record)
+                    # Last parameter True is upsert which inserts a new record if it doesnt exists or replaces current if found
+                    collection.update_one({"_id": find_id}, {"$set": record}, True)
+                else:
+                    # If no key property is available just insert the record as it is
+                    collection.insert_one(record)
+            except:
+                logger.warn(f"Failed to upsert record: {record}")
 
-        logger.info(f"Uploaded {len(to_upsert)} records into {collection}")
+        logger.info(f"Uploaded {len(to_upsert)} records into {collection.full_name}")
         record_batch.last_executed_time = datetime.now()
 
 
 def try_delete(collection, _id):
     try:
         collection.delete_one({"_id": _id})
+        logger.info(f"Deleted record with _id: {_id} from {collection.full_name}")
     except Exception as e:
-        logger.warn(f'Failed to delete record with _id: {_id}. {e}')
+        logger.warn(f"Failed to delete record with _id: {_id}. {e}")
 
 
 def persist_messages(collection, messages: io.TextIOWrapper, record_batch: RecordBatch, hard_delete: bool):
@@ -122,7 +126,10 @@ def persist_messages(collection, messages: io.TextIOWrapper, record_batch: Recor
                 if len(kps) > 1:
                     logger.warn(f'Multiple key_properties found ({",".join(kps)}).'
                                 f' Only `{kps[0]}` will be considered.')
-                _id = rec[kps[0]].strip()
+                elif len(kps) == 0:
+                    logger.debug(f"key_properties not found for stream: {stream}")
+
+                _id = rec[kps[0]]
                 rec['_id'] = _id
             except:
                 _id = None
@@ -133,7 +140,7 @@ def persist_messages(collection, messages: io.TextIOWrapper, record_batch: Recor
                         try_delete(collection, _id)
                 else:
                     rec.pop('_sdc_deleted_at', None)
-                    logger.info(f'record is {rec}')
+                    logger.info(f"record is {rec}")
                     record_batch.append(rec)
             else:
                 record_batch.append(rec)
@@ -180,11 +187,13 @@ def get_connection_string(config: dict):
         config: DB config
     Returns: A MongoClient connection string
     """
-    srv = config.get('srv') == 'true'
+    srv = config.get('srv', False)
 
     # Default SSL verify mode to true, give option to disable
-    verify_mode = config.get('verify_mode', 'true') == 'true'
-    use_ssl = config.get('ssl') == 'true'
+    verify_mode = config.get('verify_mode', True)
+    use_ssl = config.get('ssl', False)
+
+    direct_connection = config.get('direct_connection', False)
 
     connection_query = {
         'readPreference': 'secondaryPreferred',
@@ -197,6 +206,9 @@ def get_connection_string(config: dict):
     if use_ssl:
         connection_query['tls'] = 'true'
 
+    if direct_connection:
+        connection_query['directConnection'] = 'true'
+
     # NB: "sslAllowInvalidCertificates" must ONLY be supplied if `SSL` is true.
     if not verify_mode and use_ssl:
         connection_query['tlsAllowInvalidCertificates'] = 'true'
@@ -226,7 +238,7 @@ def main():
     target_collection = config.get("target_collection")
     hard_delete = config.get("hard_delete", False)
 
-    client = pymongo.MongoClient(connection_string, connectTimeoutMS=2000, retryWrites=True)
+    client = pymongo.MongoClient(connection_string)
     db = client[db_name]
     collection = db[target_collection]
     record_batch = RecordBatch(config)
